@@ -1,14 +1,15 @@
-#This file is part of Tryton.  The COPYRIGHT file at the top level of
-#this repository contains the full copyright notices and license terms.
+# This file is part of Tryton.  The COPYRIGHT file at the top level of
+# this repository contains the full copyright notices and license terms.
 from decimal import Decimal
 from trytond.model import ModelView, ModelSQL, fields
 from trytond.wizard import Wizard, StateAction
 from trytond import backend
 from trytond.pyson import Eval, PYSONEncoder
 from trytond.transaction import Transaction
-from trytond.pool import Pool
+from trytond.pool import Pool, PoolMeta
 
-__all__ = ['Line', 'MoveLine', 'OpenAccount']
+__all__ = ['Line', 'Move', 'MoveLine', 'OpenAccount']
+__metaclass__ = PoolMeta
 
 
 class Line(ModelSQL, ModelView):
@@ -26,7 +27,7 @@ class Line(ModelSQL, ModelView):
     company = fields.Function(fields.Many2One('company.company', 'Company'),
         'on_change_with_company')
     account = fields.Many2One('analytic_account.account', 'Account',
-            required=True, select=True, domain=[
+        required=True, select=True, domain=[
             ('type', '!=', 'view'),
             ['OR',
                 ('company', '=', None),
@@ -131,6 +132,24 @@ class Line(ModelSQL, ModelView):
                 (self.account.rec_name,))
 
 
+class Move:
+    __name__ = 'account.move'
+
+    def cancel(self, default=None):
+        'Reverse credit/debit of analytic lines'
+        pool = Pool()
+        AnalyticLine = pool.get('analytic_account.line')
+        cancel_move = super(Move, self).cancel(default)
+        analytic_lines = []
+        for line in cancel_move.lines:
+            for analytic_line in line.analytic_lines:
+                analytic_line.debit, analytic_line.credit = (
+                    analytic_line.credit, analytic_line.debit)
+                analytic_lines.append(analytic_line)
+        AnalyticLine.save(analytic_lines)
+        return cancel_move
+
+
 class MoveLine(ModelSQL, ModelView):
     __name__ = 'account.move.line'
     analytic_lines = fields.One2Many('analytic_account.line', 'move_line',
